@@ -15,7 +15,7 @@ import matplotlib.pyplot as plt
 import matplotlib
 import IPython
 import itk
-from ._to_itk import to_itk_image
+from ._transform_types import to_itk_image
 
 @widgets.register
 class LineProfiler(Viewer):
@@ -24,8 +24,8 @@ class LineProfiler(Viewer):
     _model_name = Unicode('LineProfilerModel').tag(sync=True)
     _view_module = Unicode('itk-jupyter-widgets').tag(sync=True)
     _model_module = Unicode('itk-jupyter-widgets').tag(sync=True)
-    _view_module_version = Unicode('^0.16.3').tag(sync=True)
-    _model_module_version = Unicode('^0.16.3').tag(sync=True)
+    _view_module_version = Unicode('^0.17.0').tag(sync=True)
+    _model_module_version = Unicode('^0.17.0').tag(sync=True)
     point1 = NDArray(dtype=np.float64, default_value=np.zeros((3,), dtype=np.float64),
                 help="First point in physical space that defines the line profile")\
             .tag(sync=True, **array_serialization)\
