@@ -12,17 +12,21 @@ import time
 import itk
 import numpy as np
 import ipywidgets as widgets
-from traitlets import CBool, CFloat, Unicode, CaselessStrEnum, Tuple, List, TraitError, validate
+from traitlets import CBool, CFloat, Unicode, CaselessStrEnum, TraitError, validate
 from ipydatawidgets import NDArray, array_serialization, shape_constraints
-from .trait_types import ITKImage, itkimage_serialization
+from .trait_types import ITKImage, PointSetList, PolyDataList, itkimage_serialization, polydata_list_serialization
 try:
     import ipywebrtc
     ViewerParent = ipywebrtc.MediaStream
 except ImportError:
     ViewerParent = widgets.DOMWidget
+import matplotlib
+import colorcet
 
 from . import cm
 
+from IPython.core.debugger import set_trace
+
 
 COLORMAPS = ("2hot",
     "Asymmtrical Earth Tones (6_21b)",
@@ -170,14 +174,11 @@ class Viewer(ViewerParent):
     _model_name = Unicode('ViewerModel').tag(sync=True)
     _view_module = Unicode('itk-jupyter-widgets').tag(sync=True)
     _model_module = Unicode('itk-jupyter-widgets').tag(sync=True)
-    _view_module_version = Unicode('^0.16.3').tag(sync=True)
-    _model_module_version = Unicode('^0.16.3').tag(sync=True)
+    _view_module_version = Unicode('^0.17.0').tag(sync=True)
+    _model_module_version = Unicode('^0.17.0').tag(sync=True)
     image = ITKImage(default_value=None, allow_none=True, help="Image to visualize.").tag(sync=False, **itkimage_serialization)
     rendered_image = ITKImage(default_value=None, allow_none=True).tag(sync=True, **itkimage_serialization)
     _rendering_image = CBool(default_value=False, help="We are currently volume rendering the image.").tag(sync=True)
-    ui_collapsed = CBool(default_value=False, help="Collapse the built in user interface.").tag(sync=True)
-    annotations = CBool(default_value=True, help="Show annotations.").tag(sync=True)
-    mode = CaselessStrEnum(('x', 'y', 'z', 'v'), default_value='v', help="View mode: x: x plane, y: y plane, z: z plane, v: volume rendering").tag(sync=True)
     interpolation = CBool(default_value=True, help="Use linear interpolation in slicing planes.").tag(sync=True)
     cmap = Unicode('Viridis (matplotlib)').tag(sync=True)
     shadow = CBool(default_value=True, help="Use shadowing in the volume rendering.").tag(sync=True)
@@ -202,10 +203,38 @@ class Viewer(ViewerParent):
             help="We are downsampling the image to meet the size limits.").tag(sync=True)
     _reset_crop_requested = CBool(default_value=False,
             help="The user requested a reset of the roi.").tag(sync=True)
+    point_sets = PointSetList(default_value=None, allow_none=True, help="Point sets to visualize").tag(sync=True, **polydata_list_serialization)
+    point_set_colors = NDArray(dtype=np.float32, default_value=np.zeros((0, 3), dtype=np.float64),
+                    help="RGB colors for the points sets")\
+                .tag(sync=True, **array_serialization)\
+                .valid(shape_constraints(None, 3))
+    geometries = PolyDataList(default_value=None, allow_none=True, help="Geometries to visualize").tag(sync=True, **polydata_list_serialization)
+    geometry_colors = NDArray(dtype=np.float32, default_value=np.zeros((0, 3), dtype=np.float64),
+                    help="RGB colors for the geometries")\
+                .tag(sync=True, **array_serialization)\
+                .valid(shape_constraints(None, 3))
+    ui_collapsed = CBool(default_value=False, help="Collapse the built in user interface.").tag(sync=True)
+    rotate = CBool(default_value=False, help="Rotate the camera around the scene.").tag(sync=True)
+    annotations = CBool(default_value=True, help="Show annotations.").tag(sync=True)
+    mode = CaselessStrEnum(('x', 'y', 'z', 'v'), default_value='v', help="View mode: x: x plane, y: y plane, z: z plane, v: volume rendering").tag(sync=True)
 
 
     def __init__(self, **kwargs):
+        if 'geometry_colors' in kwargs:
+            proposal = { 'value': kwargs['geometry_colors'] }
+            color_array = self._validate_geometry_colors(proposal)
+            kwargs['geometry_colors'] = color_array
+        self.observe(self._on_geometries_changed, ['geometries'])
+        if 'point_set_colors' in kwargs:
+            proposal = { 'value': kwargs['point_set_colors'] }
+            color_array = self._validate_point_set_colors(proposal)
+            kwargs['point_set_colors'] = color_array
+        self.observe(self._on_point_sets_changed, ['point_sets'])
+
         super(Viewer, self).__init__(**kwargs)
+
+        if not self.image:
+            return
         dimension = self.image.GetImageDimension()
         largest_region = self.image.GetLargestPossibleRegion()
         size = largest_region.GetSize()
@@ -230,7 +259,6 @@ class Viewer(ViewerParent):
                     self._downsampling = True
         if self._downsampling:
             self.extractor = itk.ExtractImageFilter.New(self.image)
-            self.extractor.InPlaceOn()
             self.shrinker = itk.BinShrinkImageFilter.New(self.extractor)
         self._update_rendered_image()
         if self._downsampling:
@@ -239,7 +267,6 @@ class Viewer(ViewerParent):
         self.observe(self._on_reset_crop_requested, ['_reset_crop_requested'])
         self.observe(self.update_rendered_image, ['image'])
 
-    @debounced(delay_seconds=1.5, method=True)
     def _on_roi_changed(self, change=None):
         if self._downsampling:
             self._update_rendered_image()
@@ -284,6 +311,7 @@ class Viewer(ViewerParent):
             f()
         self._rendering_image = True
 
+
         if self._downsampling:
             dimension = self.image.GetImageDimension()
             index = self.image.TransformPhysicalPointToIndex(self.roi[0][:dimension])
@@ -348,6 +376,46 @@ class Viewer(ViewerParent):
             raise TraitError('Invalid colormap')
         return value
 
+    @validate('geometry_colors')
+    def _validate_geometry_colors(self, proposal):
+        value = proposal['value']
+        n_colors = 0
+        if self.geometries:
+            n_colors = len(self.geometries)
+        result = np.zeros((n_colors, 3), dtype=np.float32)
+        for index, color in enumerate(value):
+            result[index,:] = matplotlib.colors.to_rgb(color)
+        if len(value) < n_colors:
+            for index in range(len(value), n_colors):
+                color = colorcet.glasbey[index % len(colorcet.glasbey)]
+                result[index,:] = matplotlib.colors.to_rgb(color)
+        return result
+
+    def _on_geometries_changed(self, change=None):
+        # Make sure we have a sufficient number of colors
+        old_colors = self.geometry_colors
+        self.geometry_colors = old_colors
+
+    @validate('point_set_colors')
+    def _validate_point_set_colors(self, proposal):
+        value = proposal['value']
+        n_colors = 0
+        if self.point_sets:
+            n_colors = len(self.point_sets)
+        result = np.zeros((n_colors, 3), dtype=np.float32)
+        for index, color in enumerate(value):
+            result[index,:] = matplotlib.colors.to_rgb(color)
+        if len(value) < n_colors:
+            for index in range(len(value), n_colors):
+                color = colorcet.glasbey[index % len(colorcet.glasbey)]
+                result[index,:] = matplotlib.colors.to_rgb(color)
+        return result
+
+    def _on_point_sets_changed(self, change=None):
+        # Make sure we have a sufficient number of colors
+        old_colors = self.point_set_colors
+        self.point_set_colors = old_colors
+
     def roi_region(self):
         """Return the itk.ImageRegion corresponding to the roi."""
         dimension = self.image.GetImageDimension()
@@ -374,36 +442,90 @@ class Viewer(ViewerParent):
         return tuple(slices)
 
 
-def view(image, ui_collapsed=False, annotations=True, interpolation=True,
-        cmap=cm.viridis, mode='v', shadow=True, slicing_planes=False,
-        gradient_opacity=0.22, **kwargs):
-    """View the image.
+def view(image=None,
+        gradient_opacity=0.22, cmap=cm.viridis, slicing_planes=False,
+        select_roi=False, shadow=True, interpolation=True,
+        point_sets=[], point_set_colors=[], #point_set_opacities=[], point_set_sizes=[],
+        geometries=[], geometry_colors=[],
+        ui_collapsed=False, rotate=False, annotations=True, mode='v',
+        **kwargs):
+    """View the image and/or point sets and/or geometries.
 
-    Creates and returns an ipywidget to visualize the image.
+    Creates and returns an ipywidget to visualize an image, and/or point sets
+    and/or geometries .
 
     The image can be 2D or 3D.
 
     The type of the image can be an numpy.array, itk.Image,
-    vtk.vtkImageData, imglyb.ReferenceGuardingRandomAccessibleInterval, or
-    something that is NumPy array-like, e.g. a Dask array.
+    vtk.vtkImageData, pyvista.UniformGrid, imglyb.ReferenceGuardingRandomAccessibleInterval,
+    or a NumPy array-like, e.g. a Dask array.
+
+    A point set or a sequence of points sets can be visualized. The type of the
+    point set can be an numpy.array (Nx3 array of point positions).
+
+    A geometry or a sequence of geometries can be visualized. The type of the
+    geometry can be an itk.Mesh.
 
     Parameters
     ----------
+
+    Images
+    ^^^^^^
+
     image : array_like, itk.Image, or vtk.vtkImageData
         The 2D or 3D image to visualize.
 
-    ui_collapsed : bool, optional, default: False
-        Collapse the native widget user interface.
+    gradient_opacity: float, optional, default: 0.22
+        Gradient opacity for the volume rendering, in the range (0.0, 1.0].
 
-    annotations : bool, optional, default: True
-        Display annotations describing orientation and the value of a
-        mouse-position-based data probe.
+    cmap: string, optional, default: 'Viridis (matplotlib)'
+        Colormap. Some valid values available at itkwidgets.cm.*
+
+    slicing_planes: bool, optional, default: False
+        Enable slicing planes on the volume rendering.
+
+    select_roi: bool, optional, default: False
+        Enable an interactive region of interest widget for the image.
 
     interpolation: bool, optional, default: True
         Linear as opposed to nearest neighbor interpolation for image slices.
 
-    cmap: string, optional, default: 'Viridis (matplotlib)'
-        Colormap. Some valid values available at itkwidgets.cm.*
+    shadow: bool, optional, default: True
+        Use shadowing in the volume rendering.
+
+    Point Sets
+    ^^^^^^^^^^
+
+    point_sets: point set, or sequence of point sets, optional
+        The point sets to visualize.
+
+    point_set_colors: list of RGB colors, optional
+        Colors for the N geometries. See help(matplotlib.colors) for
+        specification. Defaults to the Glasbey series of categorical colors.
+
+    Geometries
+    ^^^^^^^^^^
+
+    geometries: geometries, or sequence of geometries, optional
+        The geometries to visualize.
+
+    geometry_colors: list of RGB colors, optional
+        Colors for the N geometries. See help(matplotlib.colors) for
+        specification. Defaults to the Glasbey series of categorical colors.
+
+    General Interface
+    ^^^^^^^^^^^^^^^^^
+
+    ui_collapsed : bool, optional, default: False
+        Collapse the native widget user interface.
+
+    rotate : bool, optional, default: False
+        Continuously rotate the camera around the scene in volume rendering
+        mode.
+
+    annotations : bool, optional, default: True
+        Display annotations describing orientation and the value of a
+        mouse-position-based data probe.
 
     mode: 'x', 'y', 'z', or 'v', optional, default: 'v'
         Only relevant for 3D images.
@@ -413,17 +535,6 @@ def view(image, ui_collapsed=False, annotations=True, interpolation=True,
             'z': z-plane
             'v': volume rendering
 
-    shadow: bool, optional, default: True
-        Use shadowing in the volume rendering.
-
-    slicing_planes: bool, optional, default: False
-        Enable slicing planes on the volume rendering.
-
-    gradient_opacity: float, optional, default: 0.2
-        Gradient opacity for the volume rendering, in the range (0.0, 1.0].
-
-    select_roi: bool, optional, default: False
-        Enable an interactive region of interest widget for the image.
 
     Other Parameters
     ----------------
@@ -444,8 +555,11 @@ def view(image, ui_collapsed=False, annotations=True, interpolation=True,
         the visualization or retrieve values created by interacting with the
         widget.
     """
-    viewer = Viewer(image=image, ui_collapsed=ui_collapsed,
-            annotations=annotations, interpolation=interpolation, cmap=cmap,
-            mode=mode, shadow=shadow, slicing_planes=slicing_planes,
-            gradient_opacity=gradient_opacity, **kwargs)
+
+    viewer = Viewer(image=image, interpolation=interpolation, cmap=cmap, shadow=shadow,
+            select_roi=select_roi, slicing_planes=slicing_planes, gradient_opacity=gradient_opacity,
+            point_sets=point_sets, point_set_colors=point_set_colors,
+            geometries=geometries, geometry_colors=geometry_colors,
+            rotate=rotate, ui_collapsed=ui_collapsed, annotations=annotations, mode=mode,
+             **kwargs)
     return viewer
